// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
extern "C" {
  #include <archive.h>
  #include <archive_entry.h>
}
#include "asset/IAssetLoader.hpp"

/**
 * Method invoked by the libarchive internals to read bytes from the archive
 * file pointer.
 * 
 * @param archive Archive requesting the read. 
 * @param data Data pointer passed to the archive.
 * @param buffer Pointer to where the buffer pointer should be stored.
 * @return Count of bytes read.
 */
ssize_t assetLoaderArchiveRead(
  struct archive *archive,
  void *data,
  const void **buffer
);

/**
 * Method invoked by the libarchive internals to seek the archive file pointer.
 * 
 * @param archive Archive requesting the seek.
 * @param data Data pointer passed to the archive.
 * @param offset Offset to seek to.
 * @param whence Whence to seek from.
 * @return The new offset.
 */
int64_t assetLoaderArchiveSeek(
  struct archive *archive,
  void *data,
  int64_t offset,
  int32_t whence
);

/**
 * Method invoked by the libarchive internals to open the archive file pointer.
 * 
 * @param archive Archive requesting the open.
 * @param data Data pointer passed to the archive.
 * @return 0 if success, otherwise for failure.
 */
int32_t assetLoaderArchiveOpen(struct archive *a, void *data);

/**
 * Method invoked by the libarchive internals to close the archive file pointer.
 * 
 * @param archive Archive requesting the close.
 * @param data Data pointer passed to the archive.
 * @return 0 if success, otherwise for failure.
 */
int32_t assetLoaderArchiveClose(struct archive *a, void *data);

namespace Dawn {
  class AssetLoader : public IAssetLoader {
    protected:
      struct archive *assetArchive = nullptr;
      struct archive_entry *assetArchiveEntry = nullptr;
      size_t position;

    public:
      uint8_t buffer[ASSET_LOADER_BUFFER_SIZE];
      FILE *assetArchiveFile = nullptr;
      
      AssetLoader(std::string filename);
      void open() override;
      int32_t close() override;
      size_t read(uint8_t *buffer, size_t size) override;
      size_t getSize() override;
      size_t skip(size_t n) override;
      void rewind() override;
      size_t getPosition() override;
      ~AssetLoader();
  };
}