// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "assert/assertgl.hpp"
#include "display/mesh/_Mesh.hpp"
#include "dawnopengl.hpp"
#include "assert/assert.hpp"

/** Indice that references a specific vertice */
typedef int32_t meshindice_t;

namespace Dawn {
  enum MeshDrawMode {
    MESH_DRAW_MODE_TRIANGLES = GL_TRIANGLES,
    MESH_DRAW_MODE_LINES = GL_LINES
  };

  class Mesh {
    private:

    protected:
      /** Pointer to the vertex buffer on the GPU */
      GLuint vertexBuffer = -1;
      /** Pointer to the index buffer on the GPU */
      GLuint indexBuffer = -1;
      /** Pointer to the vertex buffer on the GPU */
      GLuint vertexArray = -1;

      /** How many vertices are in the mesh */
      int32_t verticeCount = -1;
      /** How many indices are in the mesh */
      int32_t indiceCount = -1;

    public:
      /**
       * Create a new set of buffers for the mesh to use.
       * 
       * @param verticeCount How many Vertices will this buffer support.
       * @param indiceCount How many Indices will this buffer support.
       */
      void createBuffers(
        int32_t verticeCount,
        int32_t indiceCount
      );
      
      /**
       * Cleanup the buffers on a given mesh. This is useful if you intend to
       * expand the count of vertices your mesh supports.
       */
      void disposeBuffers();

      /**
       * Write vertice positions to the mesh.
       * 
       * @param pos Position, within the buffer, to write to.
       * @param vertices Array of positions to write.
       * @param len How many positions are in the array.
       */
      void bufferPositions(int32_t pos, glm::vec3 *positions, int32_t len);

      /**
       * Write vertice coordinates to the mesh.
       * 
       * @param pos Position, within the buffer, to write to.
       * @param coordinates Array of coordinates to write.
       * @param len How many coordinates are in the array.
       */
      void bufferCoordinates(int32_t pos, glm::vec2 *coordinates, int32_t len);

      /**
       * Write indices to the mesh.
       * 
       * @param pos Position, within the buffer, to write to.
       * @param indices Array of indices to write.
       * @param len How many indices are in the array.
       */
      void bufferIndices(int32_t pos, meshindice_t *indices, int32_t len);

      /**
       * Draw a primitive. Primitives are drawn by their indices.
       * 
       * @param drawMode Which drawing mode to use to draw the primitive.
       * @param start Start indice (index) to draw.
       * @param count Count of indices to draw. Use -1 to draw all.
       */
      void draw(enum MeshDrawMode drawMode, int32_t start, int32_t count);

      /**
       * Cleanup a previously initiated mesh.
       */
      ~Mesh();
  };
}